/* global WScript, Enumerator */
// -- import
var fso = WScript.createObject('Scripting.FileSystemObject');
var connection = null;

// -- utilities
function error(msg) {
  WScript.echo(msg);
  logout();
  WScript.quit();
}

function check(response) {
  if (response.status != 200) {
    error('HTTP ' + response.status + ': ' + response.responseText);
  }
}

function nameOf(path) {
  path = fso.fileExists(path) ? fso.getFile(path).path : fso.getFolder(path).path;

  var i = path.lastIndexOf('\\');

  return i < 0 ? path : path.substring(i + 1);
}

function paramPart(boundary, name, value) {
  var crlf = '\r\n';

  return '--' + boundary + crlf + 'Content-Disposition: form-data; name="' + name + '"' + crlf + crlf + value;
}

function filePart(boundary, name, mime) {
  var crlf = '\r\n';

  return '--' + boundary + crlf + 'Content-Disposition: form-data; name="file"; filename="' + name + '"' + crlf + 'Content-Type: ' + mime + crlf + crlf;
}

function param(name, value) {
  return encodeURIComponent(name) + '=' + encodeURIComponent(value);
}

function post(url, contentType, data) {
  var http = WScript.createObject('WinHttp.WinHttpRequest.5.1');

  /*
    HRESULT SetTimeouts(
      [in] long ResolveTimeout,
      [in] long ConnectTimeout,
      [in] long SendTimeout,
      [in] long ReceiveTimeout
    );  
  */
  
  // Depends on the size of the files and how fast the agorum core server is.
  // Specification in ms
  http.SetTimeouts(30000, 30000, 180000, 180000);

  http.open('POST', url);
  http.option(4) = 0x3300; // ignore SSL certificate errors
  http.setRequestHeader('Content-Type', contentType);
  http.send(data);

  return {
    status: http.status,
    responseText: http.responseText
  };
}

function create(url, session, folder, className, name) {
  var response = post(
    url + '/api/rest/object.json',
    'application/x-www-form-urlencoded',
    param('sessionId', session) + '&' + 
    param('data', '{"name":"' + name + '","classname":"' + className + '","target":"' + folder + '"}'));

  check(response);

  return eval('(' + response.responseText + ')').data.id;
}

// -- uploading
function Uploader(url, session, folder) {
  this.url = url;
  this.session = session;
  this.folder = folder;
  this.boundary = '------FormBoundary' + new Date().getTime();
}

Uploader.prototype.send = function(name, data) {
  return post(
    this.url + '/api/rest/object/upload/' + 
    create(this.url, this.session, this.folder, 'FILEOBJECT', name) + '.json?sessionId=' + this.session,
    'multipart/form-data; boundary=' + this.boundary,
    data);
};

Uploader.prototype.headers = function(name, mime) {
  return paramPart(this.boundary, 'name', name) + filePart(this.boundary, name, mime);
};

Uploader.prototype.string = function(name, mime, content) {
  return this.send(name, this.headers(name, mime) + content);
};

Uploader.prototype.stream = function(name, mime, content) {
  var stream = WScript.createObject('ADODB.Stream');

  // convert headers to binary
  stream.open();
  stream.type = 2; // text
  stream.charset = 'ASCII';
  stream.writeText(this.headers(name, mime));
  stream.saveToFile('headers.tmp', 2); // create + overwrite
  stream.close();

  stream.open();
  stream.type = 1; // binary
  stream.loadFromFile('headers.tmp');
  var headers = stream.read();
  stream.close();

  // concat headers and content
  stream.open();
  stream.type = 1; // binary
  stream.write(headers);
  stream.write(content);
  stream.saveToFile('body.tmp', 2); // create + overwrite
  stream.close();

  // send
  stream.open();
  stream.loadFromFile('body.tmp');
  stream.type = 1; // binary
  var result = this.send(name, stream.read());
  stream.close();

  // remove temporary files
  fso.deleteFile('headers.tmp');
  fso.deleteFile('body.tmp');

  return result;
};

Uploader.prototype.file = function(path) {
  var stream = WScript.createObject('ADODB.Stream');

  stream.open();
  stream.type = 1; // binary
  stream.loadFromFile(path);
  var mime = 'application/octet-stream';
  var result = stream.size > 0 ? this.stream(nameOf(path), mime, stream.read()) : this.string(nameOf(path), mime, '');
  stream.close();

  return result;
};

// -- main
function uploadTree(url, sessionId, rootId, path) {
  //WScript.echo(path);

  if (fso.folderExists(path)) {
    // create folder
    var folderId = create(url, sessionId, rootId, 'FOLDEROBJECT', nameOf(path));

    // upload files
    for (var files = new Enumerator(fso.getFolder(path).files); !files.atEnd(); files.moveNext()) {
      uploadTree(url, sessionId, folderId, files.item());
    }

    for (var folders = new Enumerator(fso.getFolder(path).subFolders); !folders.atEnd(); folders.moveNext()) {
      uploadTree(url, sessionId, folderId, folders.item());
    }
  }
  else if (fso.fileExists(path)) {
    // upload file
    check(new Uploader(url, sessionId, rootId).file(path));
  }
  else {
    error('Not found: ' + path);
  }
}

function connect(server, user, password) {
  var url = server + '/api/rest/session.json';
  var loginResponse = post(
    url,
    'application/x-www-form-urlencoded',
    param('username', user) + '&' + param('password', password)
  );
  
  var login = eval('(' + loginResponse.responseText + ')');
  if (!login || (login && login.errorCode && login.errorCode > 0)) {
    error('Login error: ' + loginResponse.responseText);
  }

  return { server: server, sessionId: login.sessionId };
}

function logout() {
  if (connection) {
    var logout = httpDelete(
      connection.server + '/api/rest/session.json',
      'application/x-www-form-urlencoded',
      param('sessionId', connection.sessionId)
    );
    connection = null;
    error('In logout: ' + logout.responseText);
  }
}

function httpDelete(url, contentType, data) {
  var http = WScript.createObject('WinHttp.WinHttpRequest.5.1');

  http.open('DELETE', url);
  http.option(4) = 0x3300; // ignore SSL certificate errors
  http.setRequestHeader('Content-Type', contentType);
  http.send(data);

  return {
    status: http.status,
    responseText: http.responseText
  };
}

//error(WScript.arguments(0) + ' - ' + WScript.arguments.length);

if (WScript.arguments.length !== 5) {
  error('Usage: cscript upload-login.js <agorum-server> <agorum-user> <user-pwaasword> <folderId> <path>');
}

connection = connect(WScript.arguments(0), WScript.arguments(1), WScript.arguments(2));

uploadTree(connection.server, connection.sessionId, WScript.arguments(3), WScript.arguments(4));

logout(connection);

